-- USER CONFIG VARS ---------

-- Change this if you want the mod to take more belts for the same distance, for balancing
local BELT_COST_MULTIPLIER = 1 --(DEFAULT (integer, cannot be floating point): 1)

-- Change this if you want the mod to take more pipe for the same distance, for balancing
local PIPE_COST_MULTIPLIER = 1 --(DEFAULT (integer, cannot be floating point): 1)

-- Change this to false if you'd like to not get the belts back when you break the pair of belts.
local SHOULD_REFUND_BELTS = true -- (DEFAULT (boolean): true)

-- Change this to false if pipes should not be refunded.
local SHOULD_REFUND_PIPES = true -- (DEFAULT (boolean): true)

-- Change this if you'd like to change what percentage of belts get returned when breaking the pair of belts. 0 means no belts, 0.5 means 50% of belts used, 2 for twice the amount used, etc. Overridden if SHOULD_REFUND_BELTS is false.
local BELT_REFUND_MULTIPLIER = 1.0 --(DEFAULT (floating point): 1.0)

-- Change this if you'd like to change what percentage of pipes get returned when breaking the pair of pipe to grounds. For valid numbers, see field above
local PIPE_REFUND_MULTIPLIER = 1.0 --(DEFAULT (floating point): 1.0)


local function getDistance(e, ioEn)
	local entity = e
	local ioEntity = ioEn
	
	return math.sqrt(math.pow(math.abs(math.floor(ioEntity.position.x) - math.floor(entity.position.x)), 2) + math.pow(math.abs(math.floor(ioEntity.position.y) - math.floor(entity.position.y)), 2)) - 1
end


local function checkCount(p, c, o)
	local player = p
	local calc = c
	local of  = o
	
	if player.get_item_count(of) < calc then
		return true
	end
	
	return false
end

local function setForce(ioEn, p)
	local ioEntity = ioEn
	local player = p
	
	if ioEntity.force.name == "neutral" then
		ioEntity.force = player.force		
	end
end



script.on_event({defines.events.on_built_entity}, 
    function(e)
        local entity = e.created_entity

		if entity.type == "underground-belt" then				  
			if entity.belt_to_ground_type == "output" then
				local ioEntity = entity.neighbours
				local distance = getDistance(entity, ioEntity)
				local calc = (math.floor(distance * BELT_COST_MULTIPLIER))
				local player = game.players[e.player_index]
			
				setForce(ioEntity, player)
				
				local belt_type = string.gsub(entity.name, "underground", "transport")
			
				if checkCount(player, calc, belt_type) then						
					player.print("Not enough " .. string.gsub(belt_type, "-", " ") .. "s in inventory to create this length.")					
					
					if not player.cursor_stack.valid_for_read then 
						player.cursor_stack.set_stack{name = entity.name, count = 1}
					elseif player.cursor_stack.name == entity.name then
						player.cursor_stack.count = player.cursor_stack.count + 1
					end
					
					entity.destroy()
													
				elseif (calc > 0) then
						player.remove_item{name = belt_type, count = calc}
				end	
					
			end
			
		elseif entity.name == "underground-pipe" then
			local player = game.players[e.player_index]
			local ioEntity = entity.neighbours[2]
			
			if not ioEntity then
				return nil
			end
			
			local distance = getDistance(entity, ioEntity)
			local calc = (math.floor(distance * PIPE_COST_MULTIPLIER))
			
			setForce(ioEntity, player)
			
			if checkCount(player, calc, "pipe") then					
				player.print("Not enough pipes in inventory to create this length.")
				
				if not player.cursor_stack.valid_for_read then 
					player.cursor_stack.set_stack{name = entity.name, count = 1}
				elseif player.cursor_stack.name == entity.name then
					player.cursor_stack.count = player.cursor_stack.count + 1
				end
				
				entity.destroy()
												
			elseif (calc > 0) then
				player.remove_item{name = "pipe", count = calc}
			end
			
		end
		
	end
)

script.on_event({defines.events.on_preplayer_mined_item},
	function(e)
		local entity = e.entity
		
		if entity.type == "underground-belt" then
			
			if (not SHOULD_REFUND_BELTS) or (BELT_REFUND_MULTIPLIER <= 0) then 
				return nil
			end						
			
			if (entity.belt_to_ground_type == "output") or (entity.belt_to_ground_type == "input") then 
				local ioEntity = entity.neighbours or entity
				local distance = getDistance(entity, ioEntity)				
				
				if (distance == 0)  or (entity.force.name == "neutral") or (ioEntity.force.name == "neutral") then
					return nil
				end
				
				ioEntity.force = "neutral"
				local calc = (math.floor((distance * BELT_COST_MULTIPLIER) * BELT_REFUND_MULTIPLIER))				
				local player = game.players[e.player_index]
				local belt_type = string.gsub(entity.name, "underground", "transport")
				
				if player.character and (calc > 0) then				
					player.insert{name = belt_type, count = calc}					
				end
				
			end
		
		elseif entity.name == "pipe" then		
			if (not SHOULD_REFUND_PIPES) or (PIPE_REFUND_MULTIPLIER <= 0) then
				return nil
			end
						
			local ioEntity = entity.neighbours[2] or entity 
			local distance = getDistance(entity, ioEntity)
			
			if (distance == 0)  or (entity.force.name == "neutral") or (ioEntity.force.name == "neutral") then
				return nil
			end
			
			ioEntity.force = "neutral"
			local calc = (math.floor((distance * PIPE_COST_MULTIPLIER) * PIPE_REFUND_MULTIPLIER))
			
			local player = game.players[e.player_index]
			
			if player.character and (calc > 0) then
				player.insert{name = "pipe", count = calc}
			end
		
		end
		
	end
)